@@ -198,7 +198,22 @@ class DownloadURLOpener(urllib.request.FancyURLopener):
     def __init__(self, channel):
         self.channel = channel
         self._auth_retry_counter = 0
-        urllib.request.FancyURLopener.__init__(self, None)
+        super().__init__()
+
+    def http_error(self, url, fp, errcode, errmsg, headers, data=None):
+        """Handle http errors.
+        Overriden to give retry=True to http_error_40{1,7}.
+        See https://github.com/python/cpython/commit/80f1b059714aeb1c6fc9f6ce1173bc8a51af7dd9
+        See python issue https://bugs.python.org/issue1368368
+        """
+        result = False
+        if errcode == 401:
+            result = self.http_error_401(url, fp, errcode, errmsg, headers, data=data, retry=True)
+        elif errcode == 407:
+            result = self.http_error_407(url, fp, errcode, errmsg, headers, data=data, retry=True)
+        if result:
+            return result
+        return super().http_error(url, fp, errcode, errmsg, headers, data=data)
 
     def http_error_default(self, url, fp, errcode, errmsg, headers):
         """
@@ -297,7 +312,7 @@ class DownloadURLOpener(urllib.request.FancyURLopener):
                 block = fp.read(bs)
             else:
                 block = fp.read(min(size - read, bs))
-            if block == "":
+            if len(block) == 0:
                 break
             read += len(block)
             tfp.write(block)
@@ -734,6 +749,7 @@ class DownloadTask(object):
 
             url = util.iri_to_url(url)
 
+            logger.info("Downloading %s", url)
             downloader = DownloadURLOpener(self.__episode.channel)
 
             # HTTP Status codes for which we retry the download
@@ -805,7 +821,10 @@ class DownloadTask(object):
             # an empty filename, resulting in an empty string here (bug 1440)
             if disposition_filename is not None and disposition_filename != '':
                 # The server specifies a download filename - try to use it
-                disposition_filename = os.path.basename(disposition_filename)
+                # filename_from_url to remove query string; see #591
+                fn, ext = util.filename_from_url(disposition_filename)
+                logger.debug("converting disposition filename '%s' to local filename '%s%s'", disposition_filename, fn, ext)
+                disposition_filename = fn + ext
                 self.filename = self.__episode.local_filename(create=True,
                         force_update=True, template=disposition_filename)
                 new_mimetype, encoding = mimetypes.guess_type(self.filename)
