@@ -55,15 +55,15 @@ class gPodderFetcher(feedcore.Fetcher):
     """
     custom_handlers = []
 
-    def fetch_channel(self, channel):
+    def fetch_channel(self, channel, max_episodes):
+        for handler in self.custom_handlers:
+            custom_feed = handler.fetch_channel(channel, max_episodes)
+            if custom_feed is not None:
+                return feedcore.Result(feedcore.CUSTOM_FEED, custom_feed)
         # If we have a username or password, rebuild the url with them included
         # Note: using a HTTPBasicAuthHandler would be pain because we need to
         # know the realm. It can be done, but I think this method works, too
         url = channel.authenticate_url(channel.url)
-        for handler in self.custom_handlers:
-            custom_feed = handler.handle_url(url)
-            if custom_feed is not None:
-                return feedcore.Result(feedcore.CUSTOM_FEED, custom_feed)
         return self.fetch(url, channel.http_etag, channel.http_last_modified)
 
     def _resolve_url(self, url):
@@ -76,10 +76,17 @@ class gPodderFetcher(feedcore.Fetcher):
     def register(cls, handler):
         cls.custom_handlers.append(handler)
 
+    @classmethod
+    def unregister(cls, handler):
+        cls.custom_handlers.remove(handler)
+
 
 # The "register" method is exposed here for external usage
 register_custom_handler = gPodderFetcher.register
 
+# The "unregister" method is exposed here for external usage
+unregister_custom_handler = gPodderFetcher.unregister
+
 # Our podcast model:
 #
 # database -> podcast -> episode -> download/playback
@@ -121,7 +128,8 @@ class PodcastEpisode(PodcastModelObject):
     # effective number of characters less than that. eCryptFS recommends
     # 140 chars, we use 120 here (140 - len(extension) - len(".partial")).
     # References: gPodder bug 1898, http://unix.stackexchange.com/a/32834
-    MAX_FILENAME_LENGTH = 120
+    MAX_FILENAME_LENGTH = 120  # without extension
+    MAX_FILENAME_WITH_EXT_LENGTH = 140 - len(".partial")  # with extension
 
     __slots__ = schema.EpisodeColumns
 
@@ -144,10 +152,6 @@ class PodcastEpisode(PodcastModelObject):
         episode.description = entry['description']
         if entry.get('description_html'):
             episode.description_html = entry['description_html']
-        # TODO: This really should be handled in podcastparser and not here.
-        elif util.is_html(entry['description']):
-            episode.description_html = entry['description']
-            episode.description = util.remove_html_tags(entry['description'])
 
         episode.total_time = entry['total_time']
         episode.published = entry['published']
@@ -446,31 +450,33 @@ class PodcastEpisode(PodcastModelObject):
                 episode_filename, ext = os.path.splitext(template)
             else:
                 episode_filename, _ = util.filename_from_url(self.url)
-            fn_template = util.sanitize_filename(episode_filename, self.MAX_FILENAME_LENGTH)
 
-            if 'redirect' in fn_template and template is None:
+            if 'redirect' in episode_filename and template is None:
                 # This looks like a redirection URL - force URL resolving!
                 logger.warn('Looks like a redirection to me: %s', self.url)
                 url = util.get_real_url(self.channel.authenticate_url(self.url))
                 logger.info('Redirection resolved to: %s', url)
                 episode_filename, _ = util.filename_from_url(url)
-                fn_template = util.sanitize_filename(episode_filename, self.MAX_FILENAME_LENGTH)
 
             # Use title for YouTube, Vimeo and Soundcloud downloads
             if (youtube.is_video_link(self.url) or
                     vimeo.is_video_link(self.url) or
                     escapist_videos.is_video_link(self.url) or
-                    fn_template == 'stream'):
-                sanitized = util.sanitize_filename(self.title, self.MAX_FILENAME_LENGTH)
-                if sanitized:
-                    fn_template = sanitized
+                    episode_filename == 'stream'):
+                episode_filename = self.title
 
             # If the basename is empty, use the md5 hexdigest of the URL
-            if not fn_template or fn_template.startswith('redirect.'):
+            if not episode_filename or episode_filename.startswith('redirect.'):
                 logger.error('Report this feed: Podcast %s, episode %s',
                         self.channel.url, self.url)
-                fn_template = hashlib.md5(self.url.encode('utf-8')).hexdigest()
+                episode_filename = hashlib.md5(self.url.encode('utf-8')).hexdigest()
 
+            # Also sanitize ext (see #591 where ext=.mp3?dest-id=754182)
+            fn_template, ext = util.sanitize_filename_ext(
+                episode_filename,
+                ext,
+                self.MAX_FILENAME_LENGTH,
+                self.MAX_FILENAME_WITH_EXT_LENGTH)
             # Find a unique filename for this episode
             wanted_filename = self.find_unique_file_name(fn_template, ext)
 
@@ -498,6 +504,9 @@ class PodcastEpisode(PodcastModelObject):
             self.download_filename = wanted_filename
             self.save()
 
+        if return_wanted_filename:
+            # return the filename, not full path
+            return self.download_filename
         return os.path.join(self.channel.save_dir, self.download_filename)
 
     def extension(self, may_call_local_filename=True):
@@ -990,9 +999,9 @@ class PodcastChannel(PodcastModelObject):
         # downloaded and that the feed does not list as downloadable anymore
         # Keep episodes that are currently being downloaded, though (bug 1534)
         if self.id is not None:
-            episodes_to_purge = (e for e in existing if
+            episodes_to_purge = [e for e in existing if
                     e.state != gpodder.STATE_DOWNLOADED and
-                    e.guid not in seen_guids and not e.downloading)
+                    e.guid not in seen_guids and not e.downloading]
 
             for episode in episodes_to_purge:
                 logger.debug('Episode removed from feed: %s (%s)',
@@ -1015,7 +1024,7 @@ class PodcastChannel(PodcastModelObject):
 
     def update(self, max_episodes=0):
         try:
-            result = self.feed_fetcher.fetch_channel(self)
+            result = self.feed_fetcher.fetch_channel(self, max_episodes)
 
             if result.status == feedcore.CUSTOM_FEED:
                 self._consume_custom_feed(result.feed, max_episodes)
@@ -1241,6 +1250,12 @@ class Model(object):
 
         return self.children
 
+    def get_podcast(self, url):
+        for p in self.get_podcasts():
+            if p.url == url:
+                return p
+        return None
+
     def load_podcast(self, url, create=True, authentication_tokens=None,
                      max_episodes=0):
         assert all(url != podcast.url for podcast in self.get_podcasts())
@@ -1270,8 +1285,9 @@ def check_root_folder_path():
     if gpodder.ui.win32:
         longest = len(root) \
             + 1 + PodcastChannel.MAX_FOLDERNAME_LENGTH \
-            + 1 + PodcastEpisode.MAX_FILENAME_LENGTH + 5  # eg. .opus
+            + 1 + PodcastEpisode.MAX_FILENAME_WITH_EXT_LENGTH
         if longest > 260:
-            return _("Warning: path to gPodder home (%(root)s) is very long and can result in failure to download files.\n" % {"root": root}) \
+            return _("Warning: path to gPodder home (%(root)s) is very long "
+                     "and can result in failure to download files.\n" % {"root": root}) \
                 + _("You're advised to set it to a shorter path.")
     return None
